import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;


/*

Modification History
--------------------

Version 1.0
	This example shows how the elements of XML data are read into a program

Version 2.0
	 This example shows how the errors in XML data are trapped
     Note that the application gets the callbacks till the parsing error is found

Version 3.0

	This example demonstrates how attributes are read in
	Two access mechanisms demonstrated
	1.by attribute index;
    2.by qualified (prefixed) name.

Version 0404.0

	This example demonstrates how namespaces are applied to elements and attributes.

*/


public class MyXMLHandler extends DefaultHandler {



    static public void main(String[] args) throws Exception {

        System.out.println("\nVersion 0404.0 of MyXMLHandler in example0404\n");

		/*Create a SAX Parser Factory*/
        SAXParserFactory parserFactory = SAXParserFactory.newInstance();

		/*enable namespacing*/
        parserFactory.setNamespaceAware(true);

		/*Obtain a SAX Parser */
        SAXParser saxParser = parserFactory.newSAXParser();

		/*XML Reader is the interface for reading an XML document using callbacks*/
        XMLReader xmlReader = saxParser.getXMLReader();

		/*set the error handler*/
		xmlReader.setErrorHandler(new MyErrorHandler());

		/*Attach ContentHandler - the callbacks like startDocument,startElement etc. are o
		  overridden by the setContentHandler to trap them into user code*/
        xmlReader.setContentHandler(new MyXMLHandler());

		try{
			/*Parse an XML document - the document is read and overridden callbacks in the MyXMLHandler are invoked*/
	        xmlReader.parse("CarParts.xml");
		}
		catch (SAXParseException saxException) {
					/* If there are errors in XML data are trapped and location is displayed*/
					System.out.println("\n\nError in CarParts.xml at line:"+saxException.getLineNumber()+"("+saxException.getColumnNumber()+")\n");
					System.out.println(saxException.toString());
				}
		catch (SAXException saxEx) {
							/* If there are errors in XML data, the detailed message of the exception is displayed*/
							System.out.println(saxEx.getMessage());

  }
    } //end of main method

    public void startDocument()
	    {
	        System.out.println("\n Start Document: -----Reading CarParts.xml with MyXMLHandler------\n");
	    }

	    public void startElement(String namespaceURI, String localName,
	                             String qualifiedName, Attributes elementAttributes)
	    {
	        System.out.println("Start Element-> "+" ["+namespaceURI+"]"+qualifiedName);

			/* Get the index of price attributes*/
			int indexOfPrice=elementAttributes.getIndex("price");

			/* If a price attribute does not exist then all the attributes are printed out*/
			/* Note that in the sample the attribute name is case sensitive so all attributes
				of carstereo get printed out*/
			if(-1==indexOfPrice)
				printAllAttributes(elementAttributes);
			else
				System.out.println("\tPrice= "+elementAttributes.getValue("price"));


	    }

	    public void endElement(String namespaceURI, String localName,
	                             String qualifiedName)
	    {
	        System.out.println("End Element-> "+" ["+namespaceURI+"]"+qualifiedName);
	    }

	    public void endDocument()
	    {
	        System.out.println("\n End Document: ----------------Finished Reading the document---------------------\n");
    	}

		public void printAllAttributes(Attributes elementAttributes)
		{
			for(int i=0;i<elementAttributes.getLength();i++)
			{
				System.out.println("\t\tAttribute: ["+elementAttributes.getURI(i)+"]"+elementAttributes.getQName(i)+
											    " = "+elementAttributes.getValue(i));
			}
		}



    static class MyErrorHandler implements ErrorHandler
	{

        public void fatalError(SAXParseException saxException)
		{
            System.out.println("Fatal Error occurred "+ saxException);

        }

        public void error(SAXParseException saxException)
		{
            System.out.println("Error occurred "+ saxException);
        }

        public void warning(SAXParseException saxException)
		{
            System.out.println("warning occurred "+ saxException);
        }
    }
}
