
import javax.xml.registry.*;
import javax.xml.registry.infomodel.*;
import java.net.*;
import java.security.*;
import java.util.*;


public class MyOrganizationPublisher {
    Connection connection = null;

    public MyOrganizationPublisher() {}

    public static void main(String[] args) {


        MyOrganizationPublisher mop = new MyOrganizationPublisher();

        mop.connect();

        mop.publishdata();
    }


    public void connect() {

        String queryURL =
		              "http://localhost:8080/registry-server/RegistryServerServlet";
		        String publishURL =
		            "http://localhost:8080/registry-server/RegistryServerServlet";

        String httpProxyHost = "";
        String httpProxyPort = "8080";
        String httpsProxyHost = "";
        String httpsProxyPort = "8080";


        Properties props = new Properties();
        props.setProperty("javax.xml.registry.queryManagerURL",
            queryURL);
        props.setProperty("javax.xml.registry.lifeCycleManagerURL",
            publishURL);
        props.setProperty("com.sun.xml.registry.http.proxyHost",
            httpProxyHost);
        props.setProperty("com.sun.xml.registry.http.proxyPort",
            httpProxyPort);
        props.setProperty("com.sun.xml.registry.https.proxyHost",
            httpsProxyHost);
        props.setProperty("com.sun.xml.registry.https.proxyPort",
            httpsProxyPort);

        try {
               ConnectionFactory factory =
               ConnectionFactory.newInstance();
            factory.setProperties(props);
            connection = factory.createConnection();
            System.out.println("Created connection to registry");
        } catch (Exception e) {
            e.printStackTrace();
           }

    }


    public void publishdata() {

        String username = "testuser";
		String password = "testuser";

        RegistryService rsrvc = null;
        BusinessLifeCycleManager bsnslfcmgr = null;
        BusinessQueryManager bqrymgr = null;

        try {
            rsrvc = connection.getRegistryService();
            bsnslfcmgr = rsrvc.getBusinessLifeCycleManager();
            bqrymgr = rsrvc.getBusinessQueryManager();
            System.out.println("Got registry service, query " +
                "manager, and life cycle manager");

            // Get authorization from the registry
            PasswordAuthentication passwdAuth =
                new PasswordAuthentication(username,
                    password.toCharArray());

            Set creds = new HashSet();
            creds.add(passwdAuth);
            connection.setCredentials(creds);
            System.out.println("Security credentials established");

            // Create organization, specify name and description
            Organization orgn =
                bsnslfcmgr.createOrganization("The Great Car Parts Company");
            InternationalString descn =
                bsnslfcmgr.createInternationalString("Suppliers of high quality car accessories");
            orgn.setDescription(descn);
            System.out.println("Description about the organization added");

            // Enter the Contact Person Name
            User primaryContact = bsnslfcmgr.createUser();
            PersonName pName = bsnslfcmgr.createPersonName("Jack");
            primaryContact.setPersonName(pName);

            // Enter primary contact phone number
            TelephoneNumber telNo = bsnslfcmgr.createTelephoneNumber();
            telNo.setNumber("(800) 555-1111");
            Collection phoneNums = new ArrayList();
            phoneNums.add(telNo);
            primaryContact.setTelephoneNumbers(phoneNums);

            // Enter primary contact email address
            EmailAddress emailAdd =
                bsnslfcmgr.createEmailAddress("jack@greatcarpartcompany.com");
            Collection emailAddresses = new ArrayList();
            emailAddresses.add(emailAdd);
            primaryContact.setEmailAddresses(emailAddresses);

            // Set primary contact for organization
            orgn.setPrimaryContact(primaryContact);
            System.out.println("Primary contact details added");

            // Set classification scheme to NAICS
            ClassificationScheme cScheme =
                bqrymgr.findClassificationSchemeByName(null, "ntis-gov:naics");

            // Create and add classification
            Classification classification = (Classification)
                bsnslfcmgr.createClassification(cScheme,
                    "Car Parts and Accessories", "123456");
            Collection classifications = new ArrayList();
            classifications.add(classification);
            orgn.addClassifications(classifications);

            System.out.println("Classification added");

            // Create services and service
            Collection services = new ArrayList();
            Service service =
                bsnslfcmgr.createService("CarPartRequest");
            InternationalString desc =
                bsnslfcmgr.createInternationalString("Order Car Parts");
            service.setDescription(desc);

            // Create service bindings
            Collection serviceBindings = new ArrayList();
            ServiceBinding binding = bsnslfcmgr.createServiceBinding();
            descn = bsnslfcmgr.createInternationalString("Binding for CarPartRequest service");
            binding.setDescription(descn);
            // Turn off address validation
            binding.setValidateURI(false);
            binding.setAccessURI("http://greatcarpartcompany.com:8080/CarPartRequest/");
            serviceBindings.add(binding);

            // Add service bindings to service
            service.addServiceBindings(serviceBindings);

            // Add service to services, then add services to organization
            services.add(service);
            orgn.addServices(services);

            System.out.println("Services added");

            // Add organization and submit to registry
            // Retrieve key if successful
            Collection orgs = new ArrayList();
            orgs.add(orgn);
            //Iterator for keys or exception
            Iterator itr;
            BulkResponse response = bsnslfcmgr.saveOrganizations(orgs);
            Collection exceptions = response.getExceptions();
            if (exceptions == null) {
                System.out.println("Organization saved");

                Collection keys = response.getCollection();
                itr = keys.iterator();
                if (itr.hasNext()) {
                    javax.xml.registry.infomodel.Key orgKey =
                        (javax.xml.registry.infomodel.Key) itr.next();
                    String id = orgKey.getId();
                    System.out.println("Organization key is " + id);
                }
            } else {
                itr = exceptions.iterator();
                Exception exception = null;
                while (itr.hasNext()) {
                    exception = (Exception) itr.next();
                    System.err.println("Failed to save because: " +
                        exception.toString());
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally  {
            // At end, close connection to registry
            if (connection != null) {
                try {
                    connection.close();
                } catch (JAXRException je) {}
            }
        }
    }
}
